@@ -1,7 +1,11 @@
+import threading
+
 import procrastinate
 from anystore.functools import weakref_cache as cache
 from anystore.logging import configure_logging, get_logger
 from anystore.util import mask_uri
+from cachetools import TTLCache, cached
+from cachetools.keys import hashkey
 from procrastinate import connector, testing, utils
 from psycopg_pool import AsyncConnectionPool, ConnectionPool
 
@@ -9,18 +13,60 @@ from openaleph_procrastinate.settings import OpenAlephSettings
 
 log = get_logger(__name__)
 
+# Thread-safe cache for get_pool function
+_pool_cache = TTLCache(maxsize=10, ttl=3600)  # 1 hour TTL
+_pool_cache_lock = threading.RLock()
 
-@cache
+
+@cached(cache=_pool_cache, lock=_pool_cache_lock, key=lambda sync=False: hashkey(sync))
 def get_pool(sync: bool | None = False) -> ConnectionPool | AsyncConnectionPool | None:
+    """
+    Create a psycopg connection pool with proper configuration for long-running workers.
+
+    CRITICAL: psycopg3 does NOT check connection health by default!
+    From the docs: "The pool doesn't actively check the state of the connections
+    held in its state. This means that [...] the application might be served a
+    connection in broken state."
+
+    Without proper configuration, workers will hang after idle periods when they
+    receive stale/dead connections from the pool. This manifests as:
+    - Workers stop processing jobs after 10-60 minutes
+    - Active PostgreSQL connections drop to 0
+    - Workers appear "hung" but are actually waiting for usable connections
+
+    Configuration rationale:
+    - max_idle=300s (5min)
+      Pool proactively closes connections before PostgreSQL kills them
+    - check=ConnectionPool.check_connection is ESSENTIAL
+      Even with max_idle, connections can die from network issues, restarts, etc.
+      Without this, workers get broken connections and hang
+    - timeout=30s prevents infinite hangs if pool can't get a connection
+    - max_lifetime=3600s forces periodic connection refresh
+
+    See: https://www.psycopg.org/psycopg3/docs/advanced/pool.html
+    """
     settings = OpenAlephSettings()
     if settings.in_memory_db:
         return
+
     if sync:
         return ConnectionPool(
-            settings.procrastinate_db_uri, min_size=1, max_size=settings.db_pool_size
+            settings.procrastinate_db_uri,
+            min_size=1,
+            max_size=settings.db_pool_size,
+            check=ConnectionPool.check_connection,
+            max_idle=300,
+            max_lifetime=3600,
+            timeout=30,
         )
     return AsyncConnectionPool(
-        settings.procrastinate_db_uri, min_size=1, max_size=settings.db_pool_size
+        settings.procrastinate_db_uri,
+        min_size=1,
+        max_size=settings.db_pool_size,
+        check=AsyncConnectionPool.check_connection,
+        max_idle=300,
+        max_lifetime=3600,
+        timeout=30,
     )
 
 
